package de.ozgcloud.eingang.fim.common.xml;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Optional;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.lang3.StringUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class DocumentHelper {

	public static Document parse(InputStream inputStream) throws ParserConfigurationException, IOException, SAXException {
		return getDocumentBuilder().parse(inputStream);
	}

	public static Document parse(File file) throws ParserConfigurationException, IOException, SAXException {
		return getDocumentBuilder().parse(file);
	}

	public static DocumentBuilder getDocumentBuilder() throws ParserConfigurationException {
		return DocumentBuilderFactory.newInstance().newDocumentBuilder();
	}

	public static String extractSchemeName(Document doc) {
		var tagParts = doc.getDocumentElement().getTagName().split(":");
		var namespacePrefix = tagParts.length < 2 ? "" : (":" + tagParts[0]);
		return doc.getDocumentElement().getAttribute("xmlns" + namespacePrefix);
	}

	public static Stream<Element> getChildNodes(NodeList childNodes) {
		return IntStream.range(0, childNodes.getLength())
				.mapToObj(childNodes::item)
				.filter(Element.class::isInstance)
				.map(Element.class::cast);
	}

	public static boolean isTextNode(Node node) {
		return node.getChildNodes().getLength() == 1 && node.getChildNodes().item(0) instanceof Text;
	}

	public static String getFieldNameWithoutNamespace(String nodeName) {
		var fieldNameParts = nodeName.split(":");
		return fieldNameParts[fieldNameParts.length - 1];
	}

	public static Optional<String> getChildElement(Element element, String tagName) {
		var nameTags = element.getElementsByTagName(tagName);
		if (nameTags.getLength() != 1) {
			return Optional.empty();
		}
		return Optional.ofNullable(nameTags.item(0).getTextContent()).map(StringUtils::trim);
	}
}