package de.ozgcloud.eingang.fim;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.core.io.ResourceLoader;

import de.ozgcloud.common.test.ReflectionTestUtils;

class FimSchemeHelperTest {

	@InjectMocks
	private FimSchemeHelper helper;
	@Mock
	private FimProperties fimProperties;
	@Mock
	private ResourceLoader resourceLoader;

	@DisplayName("Init")
	@Nested
	class TestInit {

		@BeforeEach
		void initMocks() {
			setFimSchemeAdapter(Collections.emptyList());
		}

		@DisplayName("scheme adapter catalogue")
		@Nested
		class TestInitSchemeAdapterCatalogue {

			@Mock
			private FimSchemeAdapter adapter;

			@Test
			void shouldInitByAdapter() {
				setFimSchemeAdapter(List.of(adapter));

				helper.init();

				assertThat(getSchemeAdapterCatalogue()).isNotNull().hasSize(1);
			}

			@Test
			void shouldInit() {
				helper.init();

				assertThat(getSchemeAdapterCatalogue()).isNotNull().isEmpty();
			}

			private FimSchemeAdapterCatalogue getSchemeAdapterCatalogue() {
				return ReflectionTestUtils.getField(helper, "schemeAdapterCatalogue", FimSchemeAdapterCatalogue.class);
			}
		}

		private void setFimSchemeAdapter(List<FimSchemeAdapter> adapterList) {
			ReflectionTestUtils.setField(helper, "fimSchemeAdapters", adapterList);
		}

		@Test
		void shouldInitSchemeCatalogue() {
			helper.init();

			assertThat(getSchemeCatalogue()).isNotNull();
		}

		private FimSchemeCatalogue getSchemeCatalogue() {
			return ReflectionTestUtils.getField(helper, "schemeCatalogue", FimSchemeCatalogue.class);
		}
	}

	@DisplayName("Get scheme")
	@Nested
	class TestGetScheme {

		@Mock
		private FimSchemeCatalogue schemeCatalogue;
		@Mock
		private FimScheme fimScheme;

		private String identifier = "idenditifierName";

		@BeforeEach
		void mock() {
			ReflectionTestUtils.setField(helper, "schemeCatalogue", schemeCatalogue);
			when(schemeCatalogue.get(any())).thenReturn(fimScheme);
		}

		@Test
		void shouldCallSchemeCatalogue() {
			helper.getScheme(identifier);

			verify(schemeCatalogue).get(FimSchemeIdentifier.fromString(identifier));
		}

		@Test
		void shouldReturnScheme() {
			var scheme = helper.getScheme(identifier);

			assertThat(scheme).isEqualTo(fimScheme);
		}
	}

	@DisplayName("Get default scheme")
	@Nested
	class TestGetDefaultScheme {

		@Mock
		private FimSchemeCatalogue schemeCatalogue;
		@Mock
		private FimSchemeIdentifier identifier;
		@Mock
		private FimScheme fimScheme;

		@BeforeEach
		void mock() {
			ReflectionTestUtils.setField(helper, "schemeCatalogue", schemeCatalogue);
			when(schemeCatalogue.get(any())).thenReturn(fimScheme);
		}

		@Test
		void shouldCallSchemeCatalogue() {
			helper.getDefaultScheme();

			verify(schemeCatalogue).get(FimSchemeIdentifier.fromString(UnknownSchemeAdapter.UNKNOWN_SCHEME_NAME));
		}

		@Test
		void shouldReturnScheme() {
			var scheme = helper.getDefaultScheme();

			assertThat(scheme).isEqualTo(fimScheme);
		}
	}
}