package de.ozgcloud.eingang.fim;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.FormHeaderTestFactory;

class HeaderMapperTest {

	@Spy
	@InjectMocks
	private HeaderMapper mapper;

	@DisplayName("Parse formData")
	@Nested
	class TestParseFormData {

		private final FormData formData = FormDataTestFactory.createBuilder().header(FormHeader.builder().build()).build();

		@DisplayName("on existing formName")
		@Nested
		class TestOnExistingFormName {

			@BeforeEach
			void initMocks() {
				doReturn(Optional.of(FormHeaderTestFactory.FORM_NAME)).when(mapper).getFormName(any());
			}

			@Test
			void shouldCallBuildHeader() {
				mapper.parseFormData(formData);

				verify(mapper).getFormName(FormDataTestFactory.FORM_DATA);
			}

			@Test
			void shouldReturnFormDataWithHeader() {
				var mappedFormData = mapper.parseFormData(formData);

				assertThat(mappedFormData.getHeader().getFormName()).isEqualTo(FormHeaderTestFactory.FORM_NAME);
			}
		}

		@DisplayName("on missing formName")
		@Nested
		class TestOnMissingFormName {

			@BeforeEach
			void initMocks() {
				doReturn(Optional.empty()).when(mapper).getFormName(any());
			}

			@Test
			void shouldReturnOriginFormData() {
				var mappedFormData = mapper.parseFormData(formData);

				assertThat(mappedFormData).isEqualTo(formData);
			}
		}
	}

	@DisplayName("Get formName")
	@Nested
	class TestGetFormName {

		@Test
		void shouldCallFimDataUtilToGetValue() {
			try (var helper = mockStatic(FimDataUtil.class)) {
				mapper.getFormName(FormDataTestFactory.FORM_DATA);

				helper.verify(
						() -> FimDataUtil.getValue(FormDataTestFactory.FORM_DATA, List.of("G17003529", "G05001479", "G05001480"),
								HeaderMapper.FORM_NAME_KEY));
			}
		}

		@Test
		void shouldReturnValue() {
			try (var helper = mockStatic(FimDataUtil.class)) {
				helper.when(() -> FimDataUtil.getValue(any(), any(), any())).thenReturn(Optional.of(FormHeaderTestFactory.FORM_NAME));

				var formNameOpt = mapper.getFormName(FormDataTestFactory.FORM_DATA);

				assertThat(formNameOpt).hasValue(FormHeaderTestFactory.FORM_NAME);
			}
		}
	}
}