/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;

@Component
class FormSolutionsZustaendigeStelleMapper implements FormSolutionsEngineBasedMapper {

	public static final String ZUSTAENDIGE_STELLE = "zustaendigeStelle";

	@Override
	public FormData parseFormData(FormData formData) {
		return formData.toBuilder()
				.zustaendigeStelle(buildZustaendigeStelle(formData))
				.formData(removeProcessedData(formData))
				.build();
	}

	protected ZustaendigeStelle buildZustaendigeStelle(FormData formData) {
		return ZustaendigeStelle.builder()
				.organisationseinheitenId(getZustaenigeStelle(formData))
				.build();
	}

	private String getZustaenigeStelle(FormData formData) {
		return (String) formData.getFormData().get(ZUSTAENDIGE_STELLE);
	}

	protected Map<String, Object> removeProcessedData(FormData formData) {
		var cleanedData = new LinkedHashMap<>(formData.getFormData());
		cleanedData.remove(ZUSTAENDIGE_STELLE);

		return Collections.unmodifiableMap(cleanedData);
	}
}