/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import static java.util.Objects.*;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.ozgcloud.eingang.common.formdata.FormData;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class IdentifierValueParser {

	static Map<String, String> parsePanelsData(FormData formData) {
		return new IdentifierValueParser().parsePanels(formData);
	}

	private final Map<String, String> resultMap = new LinkedHashMap<>();

	Map<String, String> parsePanels(FormData formData) {
		parse(FormSolutionsPanelMapper.getPanels(formData));
		return Collections.unmodifiableMap(resultMap);
	}

	private void parse(List<Map<String, Object>> panels) {
		if (isNull(panels)) {
			return;
		}
		for (Map<String, Object> panel : panels) {
			parse(FormSolutionsPanelMapper.getComponentList(panel));
			var identifier = (String) panel.get(FormSolutionsEngineBasedAdapter.IDENTIFIER_KEY);
			var value = (String) panel.get(FormSolutionsPanelMapper.STRING_VALUE);
			if (nonNull(identifier) && nonNull(value)) {
				 resultMap.put(identifier, value);
			}
		}
	}
}
