/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import static org.assertj.core.api.Assertions.*;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import de.ozgcloud.eingang.semantik.enginebased.afm.AfmEmpfangeneStelleMapper;
import de.ozgcloud.eingang.semantik.enginebased.afm.AfmEngineBasedMapper;

class AfmEmpfangeneStelleMapperTest {

	@InjectMocks
	private AfmEmpfangeneStelleMapper mapper;

	@Test
	void shouldDoNothingIfNoEmpfangeneStelle() {
		var formData = FormDataTestFactory.createBuilder().formData(Collections.emptyMap()).build();

		var mapped = mapper.parseFormData(formData);

		assertThat(mapped).isEqualTo(formData);
	}

	@Nested
	class TestAddControlNode {
		private FormData formData = FormDataTestFactory
				.withFormDataMaps(Map.of(AfmEmpfangeneStelleMapper.EMPFANGENESTELLE, new HashMap<String, Object>()));

		@Test
		void shouldAddControlNode() {
			var mapped = mapper.parseFormData(formData);

			assertThat(getEmpfangeneStelle(mapped.getFormData())).containsKey(AfmEngineBasedMapper.KOP_CONTROLDATA_NODENAME);
		}

		@SuppressWarnings("unchecked")
		@Test
		void shouldSetFlagToTrue() {
			var mapped = mapper.parseFormData(formData);

			assertThat((Map<String, Object>) getEmpfangeneStelle(mapped.getFormData())
					.get(AfmEngineBasedMapper.KOP_CONTROLDATA_NODENAME))
							.containsEntry(AfmEngineBasedMapper.CONTROLDATA_METADATA_PROPERTYNAME, "true");
		}

		@SuppressWarnings("unchecked")
		private Map<String, Object> getEmpfangeneStelle(Map<String, Object> formMap) {
			return (Map<String, Object>) formMap.get(AfmEmpfangeneStelleMapper.EMPFANGENESTELLE);
		}
	}

}
