/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.semantik.enginebased.afm.intelliform.IntelliFormRepresentationAdapter;

class AfmEngineBasedAdapterTest {

	@Spy
	@InjectMocks
	private AfmEngineBasedAdapter adapter;
	@Mock
	private AfmEngineBasedMapper mapper;

	@Mock
	private IntelliFormRepresentationAdapter intelliFormRepresentationAdapters;

	@DisplayName("Parse form data")
	@Nested
	class TestParseFromData {
		private final String VORGANG_NUMMER = "AAAA-1234";

		private final Map<String, Object> formDataMap = Map.of(AfmAntragstellerMapper.POSTFACH_ID, "postfachIdValue");
		private final FormData formData = FormData.builder()
				.header(FormHeader.builder()
						.vorgangNummer(VORGANG_NUMMER)
						.build()
				)
				.formData(formDataMap)
				.build();

		private final FormData processedFormData = FormData.builder()
				.formData(formDataMap)
				.build();

		@BeforeEach
		void mock() {
			ReflectionTestUtils.setField(adapter, "mappers", Collections.singletonList(mapper));

			when(mapper.parseFormData(processedFormData)).thenReturn(processedFormData);
			when(intelliFormRepresentationAdapters.adaptByRepresentations(formData)).thenReturn(processedFormData);
		}

		@Test
		void shouldCallRepresentationAdapter() {
			adapter.parseFormData(formData);

			verify(intelliFormRepresentationAdapters).adaptByRepresentations(formData);
		}

		@Test
		void shouldCallMappers() {
			adapter.parseFormData(formData);

			verify(mapper).parseFormData(processedFormData);
		}

		@Test
		void shouldRemovePostfachId() {
			var mappedFormData = adapter.parseFormData(formData);

			assertThat(mappedFormData.getFormData()).doesNotContainKey(AfmAntragstellerMapper.POSTFACH_ID);
		}

	}

}
