/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import static de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedAdapter.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsAntragstellerMapper;
import de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedAdapter;
import de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedMapper;

class FormSolutionsEngineBasedAdapterTest {

	@Spy
	@InjectMocks
	private FormSolutionsEngineBasedAdapter adapter;
	@Mock
	private FormSolutionsEngineBasedMapper mapper;

	@DisplayName("Parse formData")
	@Nested
	class TestParseFormData {

		private final FormData formData = FormDataTestFactory.create();

		@BeforeEach
		void mockMappers() {
			ReflectionTestUtils.setField(adapter, "mappers", Collections.singletonList(mapper));
		}

		@Test
		void shouldCallMappers() {
			when(mapper.parseFormData(any())).thenReturn(formData);

			adapter.parseFormData(formData);

			verify(mapper).parseFormData(formData);
		}

		@Test
		void shouldCallRemoveProcessedRawData() {
			when(mapper.parseFormData(any())).thenReturn(formData);

			adapter.parseFormData(formData);

			verify(adapter).removeProcessedData(formData);
		}

		@DisplayName("remove processed data")
		@Nested
		class TestRemoveProcessedData {

			private final Map<String, Object> formDataMap = Map.of(ASSISTANT, "testValue",
					ANLIEGEN_ID, "testValue2", KOMMUNALVERWALTUNG_ID, "testValue3",
					FormSolutionsAntragstellerMapper.POSTKORBHANDLE, "testValue4");
			private final FormData formData = FormData.builder().formData(formDataMap).build();

			@Test
			void shouldRemoveAssistant() {
				var cleanedFormData = adapter.removeProcessedData(formData);

				assertThat(cleanedFormData.getFormData()).doesNotContainKey(ASSISTANT);
			}

			@Test
			void shouldRemoveAnliegenId() {
				var cleanedFormData = adapter.removeProcessedData(formData);

				assertThat(cleanedFormData.getFormData()).doesNotContainKey(ANLIEGEN_ID);
			}

			@Test
			void shouldRemoveKommunalVerwaltungId() {
				var cleanedFormData = adapter.removeProcessedData(formData);

				assertThat(cleanedFormData.getFormData()).doesNotContainKey(KOMMUNALVERWALTUNG_ID);
			}

			@Test
			void shouldRemovePostkorbhandle() {
				var cleanedFormData = adapter.removeProcessedData(formData);

				assertThat(cleanedFormData.getFormData()).doesNotContainKey(FormSolutionsAntragstellerMapper.POSTKORBHANDLE);
			}
		}
	}
}