/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import static de.ozgcloud.eingang.common.formdata.ZustaendigeStelleTestFactory.*;
import static de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsZustaendigeStelleMapper.*;
import static org.assertj.core.api.Assertions.*;

import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;

class FormSolutionsZustaendigeStelleMapperTest {

	@Spy
	private final FormSolutionsZustaendigeStelleMapper mapper = new FormSolutionsZustaendigeStelleMapper();

	@DisplayName("Parse formData")
	@Nested
	class TestParseFormData {

		private FormData formData;

		@BeforeEach
		void setup() {
			formData = FormDataTestFactory.createBuilder()
					.formData(Map.of(
							FormDataTestFactory.SIMPLE_VALUE_KEY, FormDataTestFactory.SIMPLE_VALUE,
							FormSolutionsZustaendigeStelleMapper.ZUSTAENDIGE_STELLE, ORGANISATIONSEINHEIT_ID))
					.build();
		}

		@Test
		void shouldParseFormData() {
			var resultFormData = mapper.parseFormData(formData);

			assertThat(resultFormData.getZustaendigeStelles().getFirst().getOrganisationseinheitenId()).isEqualTo(ORGANISATIONSEINHEIT_ID);
			assertThat(resultFormData.getFormData()).doesNotContainKey(ZUSTAENDIGE_STELLE);
		}

		@Test
		@DisplayName("should process ZustaendigeStelle data only")
		void shouldNotChangeAnother() {
			var resultFormData = mapper.parseFormData(formData);

			assertThat(resultFormData).usingRecursiveComparison().ignoringFields("zustaendigeStelles", "formData").isEqualTo(formData);
			assertThat(resultFormData.getFormData())
					.containsAllEntriesOf(Map.of(FormDataTestFactory.SIMPLE_VALUE_KEY, FormDataTestFactory.SIMPLE_VALUE));
		}

		@DisplayName("build zustaendigeStelle")
		@Nested
		class TestBuildZustaendigeStelle {

			@Test
			void shouldHaveOrganisationseinheitenId() {
				var zustaendigeStelle = mapper.buildZustaendigeStelle(formData);

				assertThat(zustaendigeStelle.getOrganisationseinheitenId()).isEqualTo(ORGANISATIONSEINHEIT_ID);
			}
		}

		@DisplayName("remove processed data")
		@Nested
		class TestRemoveProcessedData {

			@Test
			void shouldRemoveZustaendigeStelle() {
				var cleanedFormData = mapper.removeProcessedData(formData);

				assertThat(cleanedFormData).doesNotContainKey(ZUSTAENDIGE_STELLE);
			}
		}
	}
}