/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.xta;

import java.util.Objects;

import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.JAXBElement;
import jakarta.xml.bind.JAXBException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.ws.client.WebServiceClientException;
import org.springframework.ws.client.support.interceptor.ClientInterceptor;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.soap.SoapMessage;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import eu.osci.ws._2014._10.transport.OriginatorsType;
import eu.osci.ws._2014._10.transport.PartyIdentifierType;
import eu.osci.ws._2014._10.transport.PartyType;

@Component
class WsHeaderAddingInterceptor implements ClientInterceptor {

	@Autowired
	private XtaCurrentIdentifierService xtaCurrentIdentifierService;

	@Override
	public boolean handleRequest(MessageContext messageContext) throws WebServiceClientException {
		var soapMessage = (SoapMessage) messageContext.getRequest();
		var header = soapMessage.getSoapHeader();

		try {
			JAXBContext context = JAXBContext.newInstance(PartyType.class);
			var marshaller = context.createMarshaller();
			marshaller.marshal(createAuthor(), header.getResult());
		} catch (JAXBException e) {
			throw new TechnicalException("Error on handling Request for adding Header.", e);
		}

		return true;
	}

	JAXBElement<PartyType> createAuthor() {
		eu.osci.ws._2014._10.transport.ObjectFactory objectFactory = new eu.osci.ws._2014._10.transport.ObjectFactory();

		PartyType partyType = new PartyType();
		PartyIdentifierType identifier = new PartyIdentifierType();
		identifier.setValue(Objects.requireNonNull(
				xtaCurrentIdentifierService.getCurrentIdentifier(),
				"Expect current identifier to be set!")
		);
		partyType.setIdentifier(identifier);

		var origin = new OriginatorsType();
		origin.setAuthor(partyType);

		return objectFactory.createAuthor(partyType);
	}

	@Override
	public boolean handleResponse(MessageContext messageContext) throws WebServiceClientException {
		return true;
	}

	@Override
	public boolean handleFault(MessageContext messageContext) throws WebServiceClientException {
		return true;
	}

	@Override
	public void afterCompletion(MessageContext messageContext, Exception ex) throws WebServiceClientException {
		// nothing to do here
	}

}
