/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.xta;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Profile;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.semantik.SemantikAdapter;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Profile({ "!itcase" })
@Log4j2
@Component
class XtaRunner implements ApplicationListener<ContextRefreshedEvent> {

	@Autowired
	private XtaService service;
	@Autowired
	private SemantikAdapter semantikAdapter;

	@Override
	public void onApplicationEvent(ContextRefreshedEvent event) {
		LOG.info("Fetching XTA Messages");
		runGetXtaMessages();
	}

	void runGetXtaMessages() {
		try {
			service.getMessages().forEach(this::processAndAcknowledge);
		} catch (RuntimeException e) {
			LOG.error("Error fetch XTA Message List.", e);
		}
	}

	private void processAndAcknowledge(@NonNull FormData formData) {
		try {
			LOG.info("Process XTA-Message '{}'.", formData.getHeader().getRequestId());
			semantikAdapter.processFormData(formData);
			service.acknowledgeReceive(XtaMessageId.from(formData.getHeader().getRequestId()));
		} catch (RuntimeException e) {
			LOG.error("Error on processing XTA-Message. Continue with next message.", e);
		}
	}

}
