/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.xta;

import static de.ozgcloud.eingang.xta.XtaMessageTestFactory.*;

import java.time.ZonedDateTime;

import de.ozgcloud.eingang.common.formdata.FormMetaDataTestFactory;

class XtaMessageMetaDataTestFactory {

	static final String MESSAGE_TYPE = "Geschaeftsgang.Geschaeftsgang.0201";
	static final String FIM_MESSAGE_TYPE = "fim.S17000652.17000652001004";
	static final ZonedDateTime ORIGIN = ZonedDateTime.parse("2022-10-29T15:45:52.4942149+02:00");
	static final String SERVICE = "urn:fim:Versammlungsanzeige:1.4";
	static final String MESSAGE_TYPE_LIST_VERSION = "1.0";

	static XtaMessageMetaData create() {
		return createBuilder().build();
	}

	static XtaMessageMetaData.XtaMessageMetaDataBuilder createBuilder() {
		return XtaMessageMetaData.builder()
				.messageId(MESSAGE_ID)
				.messageType(MESSAGE_TYPE)
				.messageTypeListVersion(MESSAGE_TYPE_LIST_VERSION)
				.origin(ORIGIN)
				.xtaIdentifier(FormMetaDataTestFactory.XTA_IDENTIFIER)
				.service(SERVICE);
	}

	static XtaMessageMetaData createFim() {
		return createFimBuilder().build();
	}

	static XtaMessageMetaData.XtaMessageMetaDataBuilder createFimBuilder() {
		return createBuilder()
				.messageType(FIM_MESSAGE_TYPE);
	}
}
