/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.xta;

import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.semantik.SemantikAdapter;

class XtaRunnerTest {

	@InjectMocks
	private XtaRunner scheduler;

	@Mock
	private XtaService service;
	@Mock
	private SemantikAdapter semantikAdapter;

	@Nested
	class TestRunGetXtaMessages {

		public static final FormData MESSAGE = FormDataTestFactory.create();

		@BeforeEach
		void init() {
			when(service.getMessages()).thenReturn(Stream.of(MESSAGE));
		}

		@Test
		void shouldCallXtaServiceGetNextMessages() {
			scheduler.runGetXtaMessages();

			verify(service).getMessages();
		}

		@Test
		void shouldHandOverFormDataToSemantikAdapter() {
			scheduler.runGetXtaMessages();

			verify(semantikAdapter).processFormData(MESSAGE);
		}

		@Test
		void shouldAcknowledgeReceive() {
			scheduler.runGetXtaMessages();

			verify(service).acknowledgeReceive(XtaMessageTestFactory.MESSAGE_ID);
		}
	}
}
